#include <string>
#include <iostream>
#include <sstream>

#include <assert.h>

#include "base/filebuffer.h"

#include "shader/shader.h"
#include "shader/shaderext.h"

using namespace std;

void Shader::create(int shaderType) {
	handle = glCreateShaderObjectARB(shaderType);
}

void Shader::setSource(const char *buffer) const {
	assert(buffer);
	glShaderSourceARB(handle, 1, &buffer, NULL);
}

void Shader::compile(void) const {
	glCompileShaderARB(handle);
}

bool Shader::createFromSource(const char *filename, int shaderType, const char *prefixString) {
	ostringstream s;
	
	//if there is a string then append it first (even though it's specified last ... )
	if (prefixString) s << prefixString;

	//if there's a file then append it last ...
	{
		FileBuffer tf;
		if (filename &&
			*filename &&
			tf.load(filename) &&
			tf.getData() &&
			tf.getSize())
		{
			s << tf.getData();
		}
	}	//FileBuffer destructor

	create(shaderType);
	setSource(s.str().c_str());
	compile();
	return true;
}


//static methods:
const char *Shader::nameForType(int shaderType) {
	switch (shaderType) {
	case GL_VERTEX_SHADER_ARB: return "Vertex";
	case GL_FRAGMENT_SHADER_ARB: return "Fragment";
	}
	return "Unknown";
}
